#include <stdio.h>
#include "game.h"
#include "screen.h"
#include "opengl.h"
#include "goatkit/goatkit.h"

enum {
	BN_START,
	BN_EXIT
};

static void bn_handler(goatkit::Widget *w, const goatkit::Event &ev, void *cls);

extern GameScreen *scr_game;

static int virt_width = 800;
static int virt_height = 600;
static goatkit::Screen ui;
static goatkit::Button *bn_start, *bn_exit;
static bool start_pending, exit_pending;
static bool stopped;

bool MenuScreen::init()
{
	ui.hide();

	int bnwidth = 200;
	int bnheight = 40;

	int xpos = (virt_width - bnwidth) / 2;
	int ypos = 100;
	int vsep = 80;

	bn_start = new goatkit::Button;
	bn_start->set_position(xpos, ypos += vsep);
	bn_start->set_size(bnwidth, bnheight);
	bn_start->set_text("Don't Panic!");
	bn_start->set_callback(goatkit::EV_CLICK, bn_handler, (void*)BN_START);
	ui.add_widget(bn_start);

	bn_exit = new goatkit::Button;
	bn_exit->set_position(xpos, ypos += vsep);
	bn_exit->set_size(bnwidth, bnheight);
	bn_exit->set_text("Panic");
	bn_exit->set_callback(goatkit::EV_CLICK, bn_handler, (void*)BN_EXIT);
	ui.add_widget(bn_exit);

	ui.set_visibility_transition(300);

	if(!(goatkit::theme = goatkit::get_theme("simple"))) {
		return false;
	}

	return true;
}

void MenuScreen::destroy()
{
}


void MenuScreen::start()
{
	ui.show();
	stopped = false;
}

void MenuScreen::stop()
{
	ui.hide();
	stopped = true;
}

void MenuScreen::draw()
{
	if(ui.get_visibility() == 0.0f) {
		if(start_pending) {
			start_pending = false;
			push_screen(scr_game);
			stop();
			return;
		}
		if(exit_pending) {
			exit_pending = false;
			game_quit();
			return;
		}
	}

	if(stopped) {
		start();
	}

	glClearColor(0, 0, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, virt_width, virt_height, 0, -1, 1);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	ui.draw();

}

void MenuScreen::reshape(int x, int y)
{
	ui.set_size(virt_width, virt_height);
}


void MenuScreen::keyboard(int key, bool pressed)
{
	if(pressed) {
		switch(key) {
		case KEY_ESC:
			exit_pending = true;
			ui.hide();
			return;

		default:
			break;
		}
	}

	ui.sysev_keyboard(key, pressed);
}

void MenuScreen::mbutton(int bn, bool pressed, int x, int y)
{
	x = x * virt_width / win_width;
	y = y * virt_height / win_height;
	ui.sysev_mouse_button(bn, pressed, x, y);
}

void MenuScreen::mmotion(int x, int y)
{
	x = x * virt_width / win_width;
	y = y * virt_height / win_height;
	ui.sysev_mouse_motion(x, y);
}

static void bn_handler(goatkit::Widget *w, const goatkit::Event &ev, void *cls)
{
	if(w == bn_start) {
		start_pending = true;
		ui.hide();
	} else if(w == bn_exit) {
		exit_pending = true;
		ui.hide();
	}
}
